/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "SubtitleSSA.h"
#include "Color.h"
#include "utility.h"

#include <map>
#include <sstream>
#include <iostream>
#include <fstream>
#include <string>
#include <iomanip>


Glib::ustring check_end_char(const Glib::ustring &str);

/*
 * clean source....
 */



/*
 *
 */
Glib::ustring SubtitleSSA::get_name()
{
	return "Sub Station Alpha";
}

/*
 *
 */
Glib::ustring SubtitleSSA::get_extension()
{
	return "ssa";
}

/*
 *
 */
bool SubtitleSSA::check(const std::string &line)
{
	static RegEx ex("^ScriptType:\\s+(v|V)+4.00([^\\+]|$)");
	
	return ex.exec(line);
}


/*
 *
 */
bool SubtitleSSA::str2bool(const std::string &str)
{
	//std::cout << "str2bool :[" << str << "]" << std::endl;

	if(str == "0")
		return false;

	return true;
}

std::string bool2str(const bool &boolean)
{
	if(boolean == false)
		return "0";
	return "-1";
}

/*
 *
 */
int str2int(const std::string &str)
{
	std::istringstream s(str);
	int i = 0;
	s >> i;

	return i;
}

/*
 *	recupere dans un tableau le format a partir de la line
 */
std::vector<std::string> build_format(const std::string &text, int column=-1, bool remove_space=false)
{
	
	std::string line;

	if(remove_space)
	{
		for(unsigned int i=7; i<text.size(); ++i)
		{
			if(text[i]==' ')
				;
			else
				line+=text[i];
		}
	}
	else
		line = text;

	std::vector<std::string> array;

	std::string::size_type s=0, e=0;

	unsigned int i=0;
	do
	{
		if(column > 0 && i+1 == column)
		{
			array.push_back(line.substr(s,std::string::npos));
			break;
		}
		e = line.find(",", s);
		
		array.push_back(line.substr(s,e-s));

		s=e+1;

		++i;
	}while(e != std::string::npos);

	return array;
}






/*
 *	construtor
 */
SubtitleSSA::SubtitleSSA(Document* doc)
: m_scriptInfo(&doc->m_scriptInfo), m_model(doc->m_subtitleModel), m_style_model(doc->m_styleModel)
{
}

/*
 *
 */
SubtitleSSA::~SubtitleSSA()
{
}

/*
 *	read subtitle file
 */
bool SubtitleSSA::open(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	debug_msg("SubtitleSSA::open > start");
	
	SubtitleFormat::open(filename, encoding);
	
	std::ifstream file(filename.c_str());
	if(!file)
	{
		throw SubtitleException("SubtitleSSA", _("I can't open this file."));
	}

	std::string line;
	
	while(!file.eof() && std::getline(file, line))
	{
		if(line.find("[Script Info]") != std::string::npos)
		{
			readScripInfo(file);
		}
		else if(line.find("[V4 Styles]") != std::string::npos)
		{
			readStyles(file);
		}
		else if(line.find("[Events]") != std::string::npos)
		{
			readEvents(file);
		}
	}

	file.close();

	debug_msg("SubtitleSSA::open > finish");
	return true;
}



bool SubtitleSSA::save(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	debug_msg("SubtitleSSA::save > start");

	SubtitleFormat::save(filename, encoding);

	std::ofstream file(filename.c_str());
	if(!file)
		throw SubtitleException("SubtitleSSA", _("I can't open this file."));



	debug_msg("SubtitleSSA::save > ScriptInfo");
	// ScriptInfo
	{
		file << utf8_to_charset("[Script Info]") << std::endl;
		file << utf8_to_charset("; This script is create by subtitleeditor ") << utf8_to_charset(VERSION) << std::endl;
		file << utf8_to_charset("; http://kitone.free.fr/subtitleeditor/") << std::endl;
	
#define CHECK(label, key) if(m_scriptInfo->key.size() > 0) \
		{ file << utf8_to_charset(label) << utf8_to_charset(m_scriptInfo->key) << std::endl; }

		m_scriptInfo->ScriptType="V4.00";

		CHECK("Title: ", Title);
		CHECK("Original Script: ", OriginalScript);
		CHECK("Original Translation: ", OriginalTranslation);
		CHECK("Original Editing: ", OriginalEditing);
		CHECK("Synch Point: ", SynchPoint);
		CHECK("Script Updated By: ", ScriptUpdatedBy);
		CHECK("Update Details: ", UpdateDetails);
		CHECK("ScriptType: ", ScriptType);
		CHECK("Collisions: ", Collisions);
		CHECK("PlayResX: ", PlayResX);
		CHECK("PlayResY: ", PlayResY);
		CHECK("Timer: ", Timer);
		CHECK("Style: ", Style);
		CHECK("Dialogue: ", Dialogue);
		CHECK("Comment: ", Comment);
		CHECK("Picture: ", Picture);
		CHECK("Sound: ", Sound);
		CHECK("Movie: ", Movie);
		
#undef CHECK

		file << std::endl;
	}

	
	debug_msg("SubtitleSSA::save > Style");
	// Style
	{
		// alignment utiliser par SSA
		// 5	-  6 -	7
		// 9	- 10 - 11
		// 1	-  2 -	3
		std::map<unsigned int, unsigned int> alignment_map;
		alignment_map[1] = 1;
		alignment_map[2] = 2;
		alignment_map[3] = 3;
		alignment_map[4] = 9;
		alignment_map[5] =10;
		alignment_map[6] =11;
		alignment_map[7] = 5;
		alignment_map[8] = 6;
		alignment_map[9] = 7;

		StyleColumnRecorder column;
		
		file << "[V4 Styles]" << std::endl;

		file << "Format: "
				<< "Name, "
				<< "Fontname, "
				<< "Fontsize, "
				
				<< "PrimaryColour, "
				<< "SecondaryColour, "
				<< "TertiaryColour, "
				<< "BackColour, "
				
				<< "Bold, "
				<< "Italic, "
				<< "BorderStyle, "
				<< "Outline, "
				<< "Shadow, "
				<< "Alignment, "
				<< "MarginL, "
				<< "MarginR, "
				<< "MarginV, "
				<< "AlphaLevel, "
				<< "Encoding" << std::endl;

		Gtk::TreeNodeChildren rows = m_style_model->children();
		
		Color primaryColour, secondaryColour, outlineColour, shadowColour;
		
		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			std::ostringstream oss;

			primaryColour.set((*it)[column.primary_colour]);
			secondaryColour.set((*it)[column.secondary_colour]);
			outlineColour.set((*it)[column.outline_colour]);
			shadowColour.set((*it)[column.shadow_colour]);

			oss << "Style: " 
				<< utf8_to_charset((*it)[column.name]) << "," 
				<< utf8_to_charset((*it)[column.font_name]) << "," 
				<< (*it)[column.font_size] << ","
			
				<< primaryColour.getRGB() << "," 
				<< secondaryColour.getRGB() << "," 
				<< outlineColour.getRGB() << "," 
				<< shadowColour.getRGB() << ","
			
				<< bool2str((*it)[column.bold]) << "," 
				<< bool2str((*it)[column.italic]) << "," 
				
				<< (*it)[column.border_style] << "," 
				<< (*it)[column.outline] << "," 
				<< (*it)[column.shadow] << "," 
				<< alignment_map[ (*it)[column.alignment] ] << "," 
				<< (*it)[column.margin_l] << "," 
				<< (*it)[column.margin_r] << "," 
				<< (*it)[column.margin_v] << "," 
				<< (*it)[column.alpha_level] << "," 
				<< (*it)[column.encoding] << std::endl; 
				
			file << oss.str();
		}

		file << std::endl;
	}

	debug_msg("SubtitleSSA::save > Event");
	// Event
	{
		file << "[Events]" << std::endl;
		// format:
		file << "Format: " <<
			"Marked, " <<
			"Start, " <<
			"End, " <<
			"Style, " <<
			"Name, " <<
			"MarginL, " <<
			"MarginR, " <<
			"MarginV, " <<
			"Effect, " <<
			"Text" << std::endl;
			
		// dialog:
		SubtitleColumnRecorder column;
		Gtk::TreeNodeChildren rows = m_model->children();
		
		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			std::ostringstream oss;

			oss << "Dialogue: "
				<< "Marked=" << 0/*(*it)[column.flag]*/ << "," 
				<< convert_time_for_save((*it)[column.start]) << "," // convertir , en .
				<< convert_time_for_save((*it)[column.end]) << "," // convertir , en .
				<< utf8_to_charset((*it)[column.style]) << ","
				<< utf8_to_charset((*it)[column.name]) << ","
				<< std::setw(4) << std::setfill('0') << (*it)[column.marginL] << ","
				<< std::setw(4) << std::setfill('0') << (*it)[column.marginR] << ","
				<< std::setw(4) << std::setfill('0') << (*it)[column.marginV] << ","
				<< utf8_to_charset((*it)[column.effect]) << ","
				<< utf8_to_charset((*it)[column.text]) << std::endl;

			file << oss.str();
		}
	}
	
	file.close();

	debug_msg("SubtitleSSA::save > finish");
	return true;
}

/*
 *	READ BLOCK
 */
	
/*
 *
 */
bool SubtitleSSA::readScripInfo(std::ifstream &file)
{
	debug_msg("SubtitleSSA::readScripInfo > start");
	
	std::string line;
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
		{
			break;
		}

#define CHECK(label, key) \
		if(line.find(label) != std::string::npos) \
		{	line.erase(0, (std::string(label)).size()); m_scriptInfo->key = check_end_char(charset_to_utf8(line)); }
//		else std::cerr << "CHECK not found: " << label << std::endl;

		CHECK("Title: ", Title);
		CHECK("Original Script: ", OriginalScript);
		CHECK("Original Translation: ", OriginalTranslation);
		CHECK("Original Editing: ", OriginalEditing);
		CHECK("Synch Point: ", SynchPoint);
		CHECK("Script Updated By: ", ScriptUpdatedBy);
		CHECK("Update Details: ", UpdateDetails);
		CHECK("ScriptType: ", ScriptType);
		CHECK("Collisions: ", Collisions);
		CHECK("PlayResX: ", PlayResX);
		CHECK("PlayResY: ", PlayResY);
		CHECK("Timer: ", Timer);
		CHECK("Style: ", Style);
		CHECK("Dialogue: ", Dialogue);
		CHECK("Comment: ", Comment);
		CHECK("Picture: ", Picture);
		CHECK("Sound: ", Sound);
		CHECK("Movie: ", Movie);

#undef CHECK
	}

	debug_msg("SubtitleSSA::readScripInfo > finish");
	return true;
}

/*
 *	read Style
 */
bool SubtitleSSA::readStyles(std::ifstream &file)
{
	debug_msg("SubtitleSSA::readStyles > start");
	
	std::vector<std::string> formats;
	std::map<std::string, unsigned int> map; 
	
	std::string line;

	// alignment utiliser par SSA
	// 5	-  6 -	7
	// 9	- 10 - 11
	// 1	-  2 -	3
	std::map<unsigned int, unsigned int> alignment_map;
	alignment_map[1] = 1;
	alignment_map[2] = 2;
	alignment_map[3] = 3;
	alignment_map[9] = 4;
	alignment_map[10]= 5;
	alignment_map[11]= 6;
	alignment_map[5] = 7;
	alignment_map[6] = 8;
	alignment_map[7] = 9;
	
	Color color;
	
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
			break;//return true;

		if(line.find("Style: ") != std::string::npos)
		{
			StyleColumnRecorder column;
			// on donne la ligne sans "Style: " = size - 7
			std::vector<std::string> fmt = build_format(line.substr(7, line.size()-7), formats.size(), false);

			Gtk::TreeIter it = m_style_model->append();

			// on supprime '*' si il existe
			(*it)[column.name]							= clean_style_name( charset_to_utf8(fmt[ map["Name"] ]) );
			
			(*it)[column.font_name]					= charset_to_utf8(fmt[ map["Fontname"] ]);
			(*it)[column.font_size]					= str2int( fmt[ map["Fontsize"] ] );

			// color
			color.set( str2int( fmt[ map["PrimaryColour"] ]), 255);
			(*it)[column.primary_colour] = color.getRGBA();

			color.set( str2int( fmt[ map["SecondaryColour"] ]), 255);
			(*it)[column.secondary_colour] = color.getRGBA();

			color.set ( str2int( fmt[ map["TertiaryColour"] ]), 255);
			(*it)[column.outline_colour] = color.getRGBA();
			
			color.set( str2int( fmt[ map["BackColour"] ]), 255);
			(*it)[column.shadow_colour] = color.getRGBA();


			
			(*it)[column.bold]							= str2bool( fmt[ map["Bold"] ] );
			(*it)[column.italic]						= str2bool( fmt[ map["Italic"] ] );
			(*it)[column.border_style]			= str2int( fmt[ map["BorderStyle"] ] );
			(*it)[column.outline]						= str2int( fmt[ map["Outline"] ] );

			(*it)[column.shadow]						= str2int( fmt[ map["Shadow"] ]);
			(*it)[column.alignment]					= alignment_map[ str2int( fmt[ map["Alignment"] ]) ];
			(*it)[column.margin_l]					= str2int( fmt[ map["MarginL"] ]);
			(*it)[column.margin_r]					= str2int( fmt[ map["MarginR"] ]);
			(*it)[column.margin_v]					= str2int( fmt[ map["MarginV"] ]);
			
			(*it)[column.alpha_level]				= str2int( fmt[ map["AlphaLevel"] ]);
			(*it)[column.encoding]					= str2int( fmt[ map["Encoding"] ]);
	
		}
		else if(line.find("Format: ") != std::string::npos)
		{
			formats = build_format(line, -1, true);

			for(unsigned int i=0; i<formats.size(); ++i)
				map[formats[i]] = i;
		}
		
	}
	debug_msg("SubtitleSSA::readStyles > finish");
	return true;
}

/*
 *
 */
bool SubtitleSSA::readEvents(std::ifstream &file)
{
	debug_msg("SubtitleSSA::readEvents > start");
	
	unsigned int num = 1;
	
	std::string line;
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
			return true;

		if(/*std::string::size_type n=*/line.find("Dialogue: ") != std::string::npos)
		{
			line.erase(0,10);

			std::vector<std::string> array = build(line, 10);

			Gtk::TreeIter it = m_model->append();
			SubtitleColumnRecorder column;
			
			(*it)[column.num] = num;
			// marked/layer
			(*it)[column.start]		= convert_time_for_subtitleeditor(array[1]);
			(*it)[column.end]			= convert_time_for_subtitleeditor(array[2]);


			(*it)[column.style]		= charset_to_utf8(clean_style_name( array[3] ));
			(*it)[column.name]		= charset_to_utf8(array[4]);
			
			(*it)[column.marginL]	= array[5];
			(*it)[column.marginR]	= array[6];
			(*it)[column.marginV]	= array[7];
			(*it)[column.effect]	= charset_to_utf8(array[8]);

			(*it)[column.text]		= check_end_char(charset_to_utf8((array[9])));

			++num;
		}
	}
	debug_msg("SubtitleSSA::readEvents > finish");
	return true;
}


/*
 *
 */
std::vector< std::string > SubtitleSSA::build(const std::string &line, unsigned int column)
{
	std::vector< std::string > array;

	std::string::size_type s=0, e=0;

	do
	{
		if(column > 0 && array.size()+1 == column)
		{
			array.push_back(line.substr(s,std::string::npos));
			break;
		}

		e = line.find(",", s);
		
		array.push_back(line.substr(s,e-s));

		s=e+1;
	}while(e != std::string::npos);
	return array;
}


/*
 *	convertir le temps utiliser par subtitle editor en tant valide pour le format SSA
 *	h:m:s,ms -> h:m:s.ms
 */
Glib::ustring SubtitleSSA::convert_time_for_save(const Glib::ustring &time)
{
	//Glib::ustring str = time;
	//Glib::ustring::size_type f = str.find(',');
	//str.replace(f, 1, ".");
	//return str;
	return time;
}

/*
 *	hours:mins:secs.msecs -> hours:mins:secs,msecs
 */
Glib::ustring SubtitleSSA::convert_time_for_subtitleeditor(const Glib::ustring &text)
{
	/*
	struct stime
	{
		int hours;
		int mins;
		int secs;
		int msecs;
	};

	stime t1;
	
	int res = sscanf(text.c_str(), "%d:%d:%d.%d",
		&t1.hours, &t1.mins, &t1.secs, &t1.msecs);
	
	if(res != 4)
		return "error";
	
	SubtitleTime time(t1.hours, t1.mins, t1.secs, t1.msecs);

	return getTime2String(time);
	*/
	if(SubtitleTime::validate(text))
		return SubtitleTime(text).str();

	return "";
}

/*
 *	hack !
 */
Glib::ustring SubtitleSSA::clean_style_name(const Glib::ustring &name)
{
	Glib::ustring str = name;
	Glib::ustring::size_type n = str.find('*');
	if(n != Glib::ustring::npos)
		str.erase(n,1);

	return str;
}

